/*
 * envGUI.java
 *
 * Created on 4 ottobre 2007, 14.54
 *
 *  aliCE (c) 2007-2008
 *  Michele Piunti
 */

package alice.cartago.examples.roomsworld;


import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import alice.cartago.*;


/**
 *
 * @author michelepiunti
 */
public class envGUI extends GUIArtifact {

	private MyFrame frame;
        
        private ArrayList<trash> trashes;
        private ArrayList<wall> walls;
        
        private HashMap<String, agentInfo> agents;
        private HashMap<String, location> checklists;
        
        private location guiInfoLoc;
        private long simtime;
        

	@OPERATION void init(){
		//initGUISystem();

		frame = new MyFrame();
		registerSource(frame.stats,"actionPerformed","stats");
		registerSource(frame, "windowClosing","closed");
		
        trashes = new ArrayList<trash>();
        walls = new ArrayList<wall>();
        agents = new HashMap<String, agentInfo>();
        checklists = new HashMap<String, location>(); 
                
		frame.setVisible(true);
        simtime=0;
        guiInfoLoc = new location(0.78, 0.95);
	}

	@OPERATION void setTitle(String title){
		frame.setTitle(title);
	}
        
        @OPERATION void refresh(){
		frame.refresh();
	}
        
        
    @LINK void newAgent(agentInfo agnt){
                log(agnt.getName() + " added in "+ agnt.getLocation());
		agents.put(agnt.getName(), agnt);
	}
        
    @LINK void moveAgent(String agName, location newloc){

		agents.get(agName).setLocation(newloc);
	}
        
    @LINK void newTrash(trash trshs){
                log("new trash added");
		trashes.add(trshs);
	}
        
    @LINK void setWalls(ArrayList<wall> wls){
            walls = wls;
            log("walls added " + walls.get(1));
	}
        
    @LINK void setCheckList(HashMap<String, location> chl){
            checklists=chl;
    }
        
    @LINK void setSimTime(long st){
            simtime = st;
    }
        
    @LINK void dropTrash(trash tr){
        for(int i=0;i<trashes.size();i++){
            if(trashes.get(i).getLocation().isEqual(tr.getLocation())){
                 trashes.remove(i);
                 break;
             }
                    
         }
     }
    
    @LINK void print(String str){
        frame.console.append(str +"\n");
    }
        

	
	class MyFrame extends JFrame {		
		
		static final long serialVersionUID = 0;
            
        JButton stats,clean;
        JPanel map,io;
        JTextArea console;
        BasicStroke basicStroke, wallStroke, checkListStroke;
                
		
		public MyFrame(){
			setTitle("RoomsWorld Open MAS (c)2007-2008  aliCE, UniBo");
			setSize(500,500);
                        
                  basicStroke = new BasicStroke(0);
                  wallStroke = new BasicStroke(2);
                  checkListStroke = new BasicStroke(4);
                  createGUI();
		}
                
        private void createGUI() {
                  io = new JPanel(new BorderLayout());
                        
                  console = new JTextArea(7, 10);
                  console.setEditable(false);   
                  JScrollPane scrollconsole = new JScrollPane(console);
                  // the map
                  map = new JPanel()
                  {
                                // overridden paint method.
                                protected void	paintComponent(Graphics g)
                                {
                                        Graphics2D g2 = (Graphics2D)g;
                                        int i;
                                        Point p;

                                        // Paint background.
                                        Rectangle	bounds	= getBounds();
                                        g.setColor(Color.LIGHT_GRAY);
                                        g.fillRect(0, 0, bounds.width, bounds.height);
                                        
                                        // paint walls
                                        g2.setColor(Color.BLACK);
                                        paintWalls(g2, bounds);

                                        //paint agents
                                        Collection<agentInfo> ags = agents.values();
                                        Iterator<agentInfo> it = ags.iterator();
                                        while(it.hasNext())                                       
                                        { 
                                            agentInfo ag = it.next();
                                            //log("paint agent" + ag + "in loc " /*+ ag.getLocation()*/); 
                                            p = onScreenLocation(ag.getLocation(), bounds);
                                            g.setColor(Color.black);	
                                            g.fillOval(p.x-3, p.y-3, 7, 7);
                                            g.drawString(ag.getName(), p.x+5, p.y-5);
                                        }  
                                        
                                        //paint log artifacts
                                        Collection<location> chls = checklists.values();
                                        Iterator<location> it2 = chls.iterator();
                                        int j = 1;
                                        while(it2.hasNext())                                       
                                        { 
                                            location loc = it2.next();
                                            //log("paint agent" + ag + "in loc " /*+ ag.getLocation()*/); 
                                            p = onScreenLocation(loc, bounds);
                                            g.setColor(Color.CYAN);	
                                            g.fillOval(p.x-3, p.y-3, 7, 7);
                                            g.drawString("AR_"+j++ , p.x-10, p.y-10);                                         
                                        }

                                        //paint trashes
                                        for(i=0; i<trashes.size(); i++)
                                        {
                                                g.setColor(Color.RED);
                                                p = onScreenLocation(trashes.get(i).getLocation(), bounds);
                                                g.fillOval(p.x-3, p.y-3, 7, 7);
                                                g.drawString("trash", p.x+5, p.y+5);
                                        }
                                        
                                        //
                                        p = onScreenLocation(guiInfoLoc , bounds);
                                        g.setColor(Color.black);	
                                        
                                        g.drawString("Sim Time: "+simtime, p.x, p.y);

                                 }
                        };
                    
                        stats = new JButton("track stats");
                        stats.addActionListener(new ActionListener(){
                           public void actionPerformed(ActionEvent e) {
                             console.append("button pressed \n");                       	
                           }                           
                        });
                        
                        clean = new JButton("Clean Console");
                        clean.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e) {
                            	console.setText(""); 
                             }              
                        });
                        io.add(scrollconsole, BorderLayout.CENTER);
                        io.add(clean, BorderLayout.EAST);

                        // compose final frame UI
                        JSplitPane globalPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, map , io  );

                        //upPane.setContinuousLayout(true);
                        //upPane.setOneTouchExpandable(true);
                        globalPane.setDividerLocation(780);
                        //globalPane.setContinuousLayout(true);
                        globalPane.setOneTouchExpandable(true);
                        
			            getContentPane().add(globalPane, BorderLayout.CENTER);
                        //panel.add(globalPane);  
                }
                
                protected void refresh() {
                   map.invalidate();
                   map.repaint();
                        
                }
                
                
                protected void paintWalls(Graphics2D g2, Rectangle bounds){
           
                    Point2D.Double from, to;
                    Line2D wall;
                    g2.setStroke(wallStroke);
                    for(int i=0;i<walls.size();i++){
                        from = onScreen2DLocation(walls.get(i).getFrom(), bounds);
                        to = onScreen2DLocation(walls.get(i).getTo(), bounds);
                        wall = new Line2D.Double(from, to);
                        g2.draw(wall);
                    }
                    g2.setStroke(basicStroke);
                }//paintWalls
                
                /**
                 *  Get the on screen location for a location in  the world.
                 */
                protected Point	onScreenLocation(location loc, Rectangle bounds)
                {
                        return new Point((int)(bounds.width*loc.getX()),
                                (int)(bounds.height*(1.0-loc.getY())));
                }
                
                protected Point2D.Double onScreen2DLocation(location loc, Rectangle bounds){
                    return new Point2D.Double((int)(bounds.width*loc.x),
                            (int)(bounds.height*(1.0-loc.y)));
                    }
            
	}//MyFrame
        
     	
}