/*
 * environment.java
 * Created on 4 ottobre 2007, 13.13
 *
 */

package alice.cartago.examples.roomsworld;

import alice.cartago.*;

import java.util.*;

/**
 *
 * @author michelepiunti
 */
@ARTIFACT_MANUAL(
		outports = {
			@OUTPORT(name = "link"),
			@OUTPORT(name = "link2")
			}
)
public class environment extends Artifact {
    
    private int N_ROOMS;
    private ArrayList<room> rooms;
    private ArrayList<wall> walls;
    private long simtime;
    private int totalTrashes;
    private HashMap<String, location> checklists;
    private HashMap<String, agentInfo> agents;
    private final location START_LOCATION = new location(.5,.5);
    
   
    /** Creates a new instance of environment */
    public environment() {
         rooms  = new ArrayList<room>();
         walls = new ArrayList<wall>();
         agents = new HashMap<String, agentInfo>();
         checklists = new HashMap<String, location>(); 
    }
    
//        /** Creates a new instance of environment */
//    public environment(int n) {
//         rooms  = new ArrayList<room>();
//         walls = new ArrayList<wall>();
//         agents = new HashMap<String, agentInfo>();
//    }
//    
    
    @OPERATION void init(int nrooms){
        N_ROOMS = nrooms;
        totalTrashes = 0;
        /*
         * 8 rooms
         */
        if(N_ROOMS==8){
            rooms.add(new  room(0.25,1.0, 0.5,0.75, 1));
            rooms.add(new  room(0.5,1.0, 0.75,0.75, 2));
            rooms.add(new  room(0.75,0.75, 1.0,0.5, 3));
            rooms.add(new  room(0.75,0.5, 1.0,0.25, 4));
            rooms.add(new  room(0.5,0.25, 0.75,0.0, 5));
            rooms.add(new  room(0.25,0.25, 0.5,0.0, 6));
            rooms.add(new  room(0.0,0.5, 0.25,0.25, 7));
            rooms.add(new  room(0.0,0.75, 0.25,0.5, 8));
        } else 
        /*
         * 4 rooms
         */  
        if (N_ROOMS==4){
            rooms.add(new  room(0.0,1.0, 0.25,0.25));
            rooms.add(new  room(0.25,1.0, 1.0,0.75));
            rooms.add(new  room(0.75,0.75, 1.0, 0.0));
            rooms.add(new  room(0.0,0.25, 0.75, 0.0));
        }  
    }
    
    @OPERATION void setLayout(){
        for(int i=0; i<rooms.size(); i++ ){
            walls.addAll( rooms.get(i).getWalls() );
        }
        try {
        	outSignal("link", new Op("setWalls", walls));
            //linkOp();
        }catch(Exception ex){
            log("linkop setWalls failed");
        }
    }
    
    
    @OPERATION void setChecklists(String ch1,location lo1, String ch2,location lo2, String ch3,location lo3,String ch4,location lo4 ) 
    {
        checklists.put(ch1, lo1);
        checklists.put(ch2, lo2);
        checklists.put(ch3, lo3);
        checklists.put(ch4, lo4);     
        try {
        	//linkTo();
        	outSignal("link",new Op("setCheckList", checklists) );
        }catch(Exception ex){
            log("linkop setCheckList failed");
        }
    }
    
    @OPERATION void setChecklists(HashMap<String, location> chlss ) 
    {
        checklists = chlss;    
        try {
            //linkTo();
        	outSignal("link", new Op("setCheckList", checklists));
        }catch(Exception ex){
            log("linkop setCheckList failed");
        }
    }
    
    @OPERATION void storeAgentPerformances(){
        Iterator<agentInfo> agInf = agents.values().iterator();
        agentInfo agentI;
        int pickedTr = 0;
        
        while(agInf.hasNext()) {
               agentI = agInf.next();
               agentI.storeEffectiveness(simtime, totalTrashes );              
               pickedTr+=agentI.getPickedTrashes();
        }
        try{
        	outSignal("link2", new Op("updateTotTrashes", simtime, pickedTr));
        	outSignal("link2", new Op("updateCosts", simtime));
            if(simtime/*totalTrashes*/>0)
                outSignal("link2", new Op("updateEffectiveness", simtime, (double)pickedTr/(double)simtime/*totalTrashes*/));
            else
            	outSignal("link2", new Op("updateEffectiveness", simtime, 0));
        }catch(Exception ex){
            log("linkop to update stats failed! with :");
            ex.printStackTrace();
        }
    }
      
    @OPERATION void getStats() {
        int tot = 0;
        Iterator<agentInfo> agInfs = agents.values().iterator();
        while(agInfs.hasNext()) {
            agentInfo agInf = agInfs.next();
            log(agInf.getName() + " - " + agInf.getEffectiveness());
            tot+=agInf.getPickedTrashes();
        }
        // signalling the total number of picked trashes (achieved goals)
        signal("stats_available",tot);
    }
    
    @OPERATION void getNTrashes() {
        int tot = 0;
        for(int i=0;i<rooms.size();i++) {
             tot+= rooms.get(i).getNTrashes();
        } 

        // signalling the total number of trashes present in rooms
        signal("trashes_available",tot);
    }
    
    @OPERATION void incTrashes() {
        Random r = new Random();
        ArrayList<room> voidrooms = new ArrayList<room>();
        for(int i=0; i<rooms.size(); ++i){
            if(rooms.get(i).getNTrashes()<1) voidrooms.add(rooms.get(i));
        }
        
        room  ro = voidrooms.get(r.nextInt(voidrooms.size()));
        trash newTR =  ro.addTrash();
        ++totalTrashes;
        log("room: "  + ro.getNo() + " now contains " + ro.getNTrashes() + " trashes");
        try {
            //linkOp();
        	outSignal("link", new Op("newTrash",newTR));
        }catch(Exception ex){
            log("linkop add trash failed");
        }
        
    }
    
    @OPERATION void getTrashes() {
        ArrayList<trash> totTrashes = new ArrayList<trash>();
        for(int i=0;i<rooms.size();i++) {
             totTrashes.addAll( rooms.get(i).getTrashes() );
        }
        signal("see_trashes",totTrashes); 
        
    }
    
    @OPERATION void join(String name) {
        
        try {
            agentInfo newAgent = new agentInfo(name);
            newAgent.setLocation(START_LOCATION);
            agents.put(name, newAgent );
            signal("joined", START_LOCATION);            
            //linkTo();
            outSignal("link", new Op("newAgent",newAgent));
            log(name + " joined environment !!! now there are n: " + agents.size() +" agents") ;
 
        }catch(Exception ex){
            log("join or linkop add agent failed...");
        }
    }
    
    @OPERATION void getNRooms() {
        signal("n_rooms", N_ROOMS);       
    }
    
    @OPERATION void getAgents() {
        signal("see_agents", agents);       
    }
    
    @OPERATION void clean(String agName, double X, double Y) {
    	log(agName + " wants to grasp a trash in: " + X + ", "+Y );
    	clean(agName, new trash(X,Y));
    }
    
    @OPERATION void clean(String agName, trash toClean) {
       try  {
           boolean ret = false;
           for(int i=0; i<rooms.size(); ++i){
              if(rooms.get(i).delTrash(toClean)) {
                    ret=true;
                    break;
              }
           }

            if(ret){
               log("trash cleaned at time: " + simtime);
               // update agent stats
               agents.get(agName).incPickedTrashes(simtime, toClean); 
               signal(agName+"cleaned", toClean); 
               //linkTo();
               outSignal("link", new Op("dropTrash", toClean));
            }else{
            	signal("fail", toClean);
            }

        }catch(Exception ex){
            log("clean trash failed");
        }
        
    }
    
    
    @OPERATION void move(String agName, Object trg) {       
        try {            
            location target = (location)trg;
            agents.get(agName).setLocation(target);           
            String signal = "movedin"+target.signalling();
            signal(signal, target);
            //log(" signal moved in:" + signal ) ;
            //linkTo();
            outSignal("link", new Op("moveAgent", agName, target));
        }catch(Exception ex){
            log("linkop move agent failed");
            ex.printStackTrace();
        }
    }
    
    @OPERATION void getLoc(String agName) {                             
        location loc = agents.get(agName).getLocation();           
        String signal = agName+"-loc";
        //log(signal);
        signal(signal, loc);           
    }
    
    
    @OPERATION void getVision(String agName, int agRoom) {
        log("requested vision by:"+ agName +" - in room: " +agRoom);
        ArrayList<trash> totTrashes = new ArrayList<trash>();

        totTrashes.addAll( rooms.get(agRoom).getTrashes() );

        Vision v = new Vision();
        v.setTrashes(totTrashes);

        signal(agName+"vision", v); 
        
        log("agent "+agName+" has discovered n: " +totTrashes.size()  + "with vision " + v);
    }
    
    @LINK void setSimTimE2(long st){
            simtime = st;
            try{
              //propagate sim time to gui visualization  
              //linkTo();
            	outSignal("link", new Op("setSimTime", st ));
            }catch(Exception e){
                log("linkop set sim time from env to gui failed");
            }
    }
    
    
}
