/*
 * location.java
 *
 * Created on 4 ottobre 2007, 13.19
 *
 *  aliCE (c) 2007-2008
 *  Michele Piunti
 */

package alice.cartago.examples.roomsworld;


import java.lang.Math;


/**
 *  The location represents a location on the world map.
 */
public class location  implements java.io.Serializable {
	
	static final long serialVersionUID = 0;
	//-------- constants --------

	/** Distance, when two locations are considered near. */
	public static final double	DEFAULT_TOLERANCE	= 0.001;

	//-------- attributes --------

	/** The x coordinate. */
	public double	x;

	/** The y coordinate. */
	public double	y;

	//-------- constructors --------


	/**
	 *  Create a new RANDOM location.
	 */
	public location()
	{
		this.x	= Math.random();
		this.y	= Math.random();
	}



	/**
	 *  Create a new location.
	 *  @param x	The x coordinate.
	 *  @param y	The y coordinate.
	 */
	public location(double x, double y)
	{
		this.x	= x;
		this.y	= y;
	}

	//-------- getter methods --------

	public double getX() {
	   return x;
	}

	public double getY() {
	   return y;
	}


	/**
	 *  Caculate how much is a location near to his location.
	 *  @return The distance.
	 */
	public double getDistance(location other)
	{
		return Math.sqrt((other.y-this.y)*(other.y-this.y)+(other.x-this.x)*(other.x-this.x));
	}



	/**
	 *  Check, if two locations are near to each other
	 *  using the default tolerance.
	 *  @return True, if two locations are near to each other.
	 */
	public boolean	isNear(location other)
	{
		return isNear(other, DEFAULT_TOLERANCE);
	}


	/**
	 *  Check, if two locations are in the same position
	 *  using the tolerance = 0.
	 *  @return True, if two locations are in the same coordinates.
	 */
	public boolean	isEqual(location other)
	{
		return isNear(other, 0.0);
	}

	/**
	 *  Check, if two locations are near to each other.
	 *  @param tolerance	The distance, when two locations are considered near.
	 *  @return True, if two locations are near to each other.
	 */
	public boolean	isNear(location other, double tolerance)
	{
		return (getDistance(other) <= tolerance);
	}

	/**
	 *  Convert the location to a string representation.
	 */
	public String	toString()
	{
		return "Location(" + x + "," + y + ")";
	}
        
        public int signalling(){
            return new Double(x+y).hashCode();
        }

        /**
         *  @return cloned location
         */
        public location clone_loc() {
            return new location(x,y);
        }
        
        public void update(double xx, double yy){
            this.x = xx;
            this.y = yy;
        }

        
}