/*
 * wall.java
 *
 * Created on 4 ottobre 2007, 14.50
 *
 *  aliCE (c) 2007-2008
 *  Michele Piunti
 */

package alice.cartago.examples.roomsworld;


import java.lang.Math;

/**
 *  wall is a rect FROM location_1 TO location_2
 * @author Michele Piunti
 */
public class wall {
	
  static final long serialVersionUID = 0;
    
  location from, to;
  
  /** 
  *  PROXIMITY = 0.1;    door closeness rate to become isNear true
  */
  public static final double DEFAULT_TOLERANCE = 0.02;
    
    /** Creates a new instance of wall */
    public wall() {
    }
    
     /**
      * Creates a new instance of wall 
      * @param fromX, fromY  (location FROM)
      * @param toX, toY  (location TO)
      */
    public wall(double fromX, double fromY, double toX, double toY) {
        this.from = new location(fromX, fromY);
        this.to = new location(toX, toY);
    }
    
    public wall(location from_loc, location to_loc){
        this.from = from_loc;
        this.to   = to_loc;
    }
    
    /**
     *  Caculate distanche between a location and the wall.
     *  @param other location
     *  @return The distance.
     */
    public double getDistance(location other)
    {
        double ret = 100;
        if(isVertical()) ret = Math.abs(other.getX()-this.to.getX());
        else if(isHorizontal()) ret = Math.abs(other.getY()-this.to.getY());
            else  try{
                double M = getM();
                double Q = getQ();
                ret =( Math.abs(other.getY()- M*other.getX() - Q))  / Math.sqrt( 1 + M*M );
                }catch(Exception exc){
 
                  ret = Math.abs(other.getX()-this.to.getX());
                }
        return ret;
    }
    

    
    /**
     * y = M * x + Q
     * @return ang coeff M of the rect between location 'from' and 'to'
     */
    public double getM() throws Exception{
        if(from.getX()==to.getX()) throw new Exception("sticazzi muro verticale");
        else return (from.getY()-to.getY())/(from.getX()-to.getX());
    }
    
    
    /**
     * y - M * x = Q
     * @return Q of the rect between location 'from' and 'to'
     */
    public double getQ() throws Exception{
       //if(from.getX()==to.getX()) throw new Exception("sticazzi muro verticale"); 
       return to.getY()- getM()*to.getX();
    }
    
    
    /**
     * get location from(x,y) 
     */
    public location getFrom(){
        return from;
    }
    
    /**
     * get location to(x,y) 
     */
    public location getTo(){
        return to;
    }
    
    /**
     * get coordinates from(x,y) to(x,y)
     */
    public double getFromX(){
        return from.getX();
    }
    
    public double getFromY(){
        return from.getY();
    }
        
    public double getToX(){
        return to.getX();
    }
    
    public double getToY(){
        return to.getY();
    }
    
    public boolean isVertical(){
        return (this.from.getX() == this.to.getX());
    }
    
     public boolean isHorizontal(){
        return (this.from.getY() == this.to.getY());
     }
     
     
    /**
    * a location is Near to the wall if its distance is less than DEFAULT_TOLERANCE
    * @param loc the location to check if it is near to the wall
    * @returns true if loc is Near to the wall
    */
    public boolean isNear(location loc){
        double distance;
        
        if((isVertical()&&((loc.getY()>=this.from.getY())&&(loc.getY()<=this.to.getY()))) ||
                (isHorizontal()&&((loc.getX()>=this.from.getX())&&(loc.getX()<=this.to.getX()))))
        distance = getDistance(loc);
        
        else distance = Math.min(loc.getDistance(from), loc.getDistance(to));
        
        return (distance < DEFAULT_TOLERANCE);
     }
    
     /**
    * a location is Near to the wall if its distance is less than limit distance
    * @param loc the location to check if it is near to the wall
    * @param dist = the limit distance
    * @returns true if loc is Near to the wall
    */
    public boolean isNear(location loc, double dist){
        double distance;
        
        if((isVertical()&&((loc.getY()>=this.from.getY())&&(loc.getY()<=this.to.getY()))) ||
                (isHorizontal()&&((loc.getX()>=this.from.getX())&&(loc.getX()<=this.to.getX()))))
        distance = getDistance(loc);
        
        else distance = Math.min(loc.getDistance(from), loc.getDistance(to));
        
        return (distance < dist );
     }
    
    
}
    
    
