/**
 * CARTAGO-JADEX-BRIDGE - Developed by aliCE team at deis.unibo.it
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package alice.c4jadex.bridge;

import alice.cartago.*;


import java.util.*;
import java.util.concurrent.locks.ReentrantLock;

public class AgentContext extends ActivityContext {

	private ICartagoContext context;
	private HashMap<String,ArtifactId> artifacts;
	private HashMap<String,SensorId> sensors;
	private SensorId defaultSensor;
	private Thread perceptHandler;
	private ReentrantLock activityLock;
	private WorkspaceId wspId;
	
	private  LinkedList<alice.cartago.Event> cartagoEventQueue;
	
	public AgentContext(String agentName, WorkspaceId wspId, ICartagoContext context) throws Exception {
		
		this.context = context;
		this.wspId = wspId;

		artifacts = new HashMap<String,ArtifactId>();
		sensors  = new HashMap<String,SensorId>();
		activityLock = new ReentrantLock();
		defaultSensor = context.linkDefaultSensor();
		cartagoEventQueue = new java.util.LinkedList<alice.cartago.Event>();
	}
	
	public WorkspaceId getWspId(){
		return wspId;
	}
	
	public ICartagoContext getContext(){
		return context;
	}
	
	public synchronized String storeArtifactId(ArtifactId aid){
		String id = aid.getName();
		artifacts.put(id, aid);
		return id;
	}

	public SensorId getDefaultSensor(){
		return defaultSensor;
	}
	
	public synchronized void storeSensorId(String sname, SensorId sid){
		sensors.put(sname, sid);
	}

	public synchronized ArtifactId getArtifactId(String name){
		return artifacts.get(name);
	}

	public synchronized SensorId getSensorId(String name){
		return sensors.get(name);
	}
        
    public ReentrantLock getLock() {
	   return activityLock;
	}
        
    public void postNewEventInSensor(String sensorN, alice.cartago.Event ev) {
		postNewEventInSensor(getSensorId(sensorN), ev);
	}

	public void postNewEventInSensor(SensorId sid, alice.cartago.Event ev) {
       
       synchronized(cartagoEventQueue){
    	   //System.out.println(">>>> Adding new artifact event: " + ev +"<<<<<");
    	   cartagoEventQueue.add(ev);
       }
	}
	
	public LinkedList<alice.cartago.Event> getEventQueue() {
	              
	    return cartagoEventQueue;
	}
	
	public void clearEvents() {
		synchronized(cartagoEventQueue){
		   cartagoEventQueue.clear();
		}
	}
	
	public void postEventRemovedFromSensor(SensorId sid, alice.cartago.Event ev){
		
	}
        
        
		


}
