/**
 * CARTAGO-JADEX-BRIDGE - Developed by aliCE team at deis.unibo.it
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
//package alice.cartago.jason;
package alice.c4jadex.bridge;


import java.util.HashMap;
import alice.cartago.*;

public class AgentContextManager {

	private AgentContext currentContext;
	private HashMap<String,AgentContext> contexts = new HashMap<String,AgentContext>();;
	
	public AgentContextManager(String agentName){
		 
		currentContext = null;
		//
		// joining default workspace
		//
               
		try{   
                    CartagoService.installNode();
		} catch (Exception ex) {
			System.err.println("[AGENT CONTEXT MANAGER] Impossible to install node in default port "
					+ ex.getLocalizedMessage() );
			//ex.printStackTrace();
		}
		try{   
                    ICartagoContext context = CartagoService.joinDefaultWorkspace(agentName);
                    WorkspaceId wspId = context.getWorkspaceId();
                    AgentContext actx = new AgentContext(agentName,wspId,context);
                    contexts.put(wspId.getName(), actx);			
                    currentContext = actx;

		} catch (Exception ex) {
			System.err.println("[AGENT CONTEXT MANAGER] Impossible to join default workspace for agent "+ agentName);
			//ex.printStackTrace();
		}
		                    
                    
	}
        
        public void register(WorkspaceId wspId, AgentContext ctx){
		synchronized (contexts){
			contexts.put(wspId.getName(), ctx);
		}
	}


	public void setCurrentContext(String wspName) throws InvalidWorkspaceException {
		synchronized (contexts){
			AgentContext ctx = contexts.get(wspName);
			if (ctx == null){
                            System.err.println("[AGENT CONTEXT MANAGER] Error: context \"" + wspName +"\" not stored.");
                            System.out.println("[AGENT CONTEXT MANAGER] actual contexts are: " + contexts.values());
			    throw new InvalidWorkspaceException();
			} else {
				currentContext = ctx;
			}
		}
	}
     
	
	public AgentContext getCurrentContext(){
		return currentContext;
	}

        	
	public synchronized AgentContext getContext(String name){
		synchronized (contexts){
                    return contexts.get(name);
                }
	}
	
}
