/*
 * EpistemicPlan.java
 *
 * Created on 17 marzo 2008, 10.03
 *
 * by Michele Piunti
 * ISTC - CNR, Rome, Italy.
 * AliCe - DEIS - UniBO, Blogna-Cesena, Italy.
 */

package alice.c4jadex.roomsworld.normal;

import alice.c4jadex.bridge.*;
import alice.c4jadex.roomsworld.navigator.*;
import alice.cartago.*;
import alice.cartago.examples.roomsworld.location;
import alice.cartago.examples.roomsworld.trash;
import alice.cartago.examples.roomsworld.Vision;
import jadex.runtime.*;

import java.util.ArrayList;
import java.util.Iterator;

/**
 *    EpistemicPlan
 * @author Michele Piunti
 */
public class EpistemicPlan extends Plan {
    private location myLoc;
    private String sensorName,agentName;
    private Perception percepts;
    private IGoal ep_action,sense;
    private int myRoom;
    
    /** Creates a new instance of EpistemicPlan */
    public EpistemicPlan() {
        sensorName = (String)getBeliefbase().getBelief("my_sensor").getFact();
        agentName = getAgentName();
    }
    
    public void body(){
        IRoomsWorldNavigator nav 
                = (IRoomsWorldNavigator)getBeliefbase().getBelief("navigator").getFact();
        myLoc = (location)getBeliefbase().getBelief("my_loc").getFact();
        myRoom = nav.getRoom(myLoc);
        
        if(getVision(myRoom)){
            IGoal clean = createGoal("clean");
            dispatchTopLevelGoal(clean);
        } else {
           IGoal selectRoom = createGoal("select_room");
           dispatchSubgoal(selectRoom);
        }

    }
    
    
    private boolean getVision( int nRoom) {
	boolean ret = false;
	
        String filter = agentName+"vision";
        ep_action = createGoal("use");
        sense = createGoal("sense");
        
	//
	try{
	    ep_action.getParameter("artifact_name").setValue("environment");
            ep_action.getParameter("sensor_name").setValue(sensorName);  
            Op getVision = new Op("getVision", agentName, nRoom-1);
            ep_action.getParameter("op").setValue(getVision);         
            dispatchSubgoalAndWait(ep_action);
            
            sense.getParameter("sensor_name").setValue(sensorName);
            sense.getParameter("filter").setValue(filter);
            //sense.getParameter("timeout").setValue(100);
            dispatchSubgoalAndWait(sense);
            percepts = (Perception)sense.getParameter("percept").getValue();
            Vision v = (Vision)percepts.getContent(0); 
            
	    ArrayList<trash> trashes = v.getTrashes();
            log(" My actual Vision has: " + trashes.size() + " trash items. Updating Beliefs.");

            if(trashes.size()>0){
		 
		  for(int i=0; i<trashes.size();i++){
		    getBeliefbase().getBeliefSet("trashes").addFact(trashes.get(i));
                    ret=true;
		  }
		  if(!ret) {
                      log(" no percepts in room: "+myRoom);                      
                  }
            }else {
                IGoal selectRoom = createGoal("select_room");
                dispatchTopLevelGoal(selectRoom);
	    }  
			
	    } catch (Exception e) {
                // throw new JasonException("Error in internal action 'roomsworld.epistemicAction': " + e, e);
                log("epistemicPlan - error in getVision \n" + e.getMessage());
                e.printStackTrace();
           }

        return ret;
	}
    
    protected void logAgentSide(String s){
        System.out.println("["+getAgentName()+"] " + s);
    }
    
    protected void log(String s){
        IGoal use = createGoal("use");
        
        use.getParameter("artifact_name").setValue("io");
        use.getParameter("sensor_name").setValue(sensorName);  
        String msg = "["+getAgentName()+"] " + s;
        Op op = new Op("print", msg );
        use.getParameter("op").setValue(op);         
        dispatchSubgoalAndWait(use);
 
        System.out.println(msg);
    }
    
}
