/*
 * joinRoomsWorldPlan.java
 *
 * Created on 4 dicembre 2007, 15.40
 *
 * by Michele Piunti
 * ISTC - CNR, Rome, Italy.
 * AliCe - DEIS - UniBO, Blogna-Cesena, Italy.
 */

package alice.c4jadex.roomsworld.normal;

import alice.c4jadex.bridge.*;
import alice.c4jadex.roomsworld.navigator.*;
import alice.cartago.*;
import alice.cartago.examples.roomsworld.location;
import jadex.runtime.*;


/**
 *    joinRoomsWorldPlan <br/>
 *   a Jadex Agent joins to the CARTAGO RoomsWorld scenario, <br/>
 *   gets his initial location and the actual number of ROOMS and <br/>
 *   store them in Beliefs
 *
 * @author Michele Piunti
 */
public class joinRoomsWorldPlan  extends Plan {
    ArtifactId aid;
    IGoal use, sense;
    Op op;
    Perception p;
    IRoomsWorldNavigator navigator; 
    
    /** Creates a new instance of makeArtifactPlan */
    public joinRoomsWorldPlan() {
        log("JOINING to CARTAGO ROOMWORLD scenario...");
    }
    
    
    public void body(){
        aid=null;
        String aName = "environment";
        String sensorName = (String)getBeliefbase().getBelief("my_sensor").getFact();
        
        //1. look up artifact
        IGoal lookup = createGoal("lookup");
        lookup.getParameter("artifact_name").setValue(aName);                
        dispatchSubgoalAndWait(lookup);
        aid = (ArtifactId)lookup.getParameter("aid").getValue();
        log(" retrieved artifact environment with a_id: " + aid);
        
        //2. Join environment and sense Initial Location
        use = createGoal("use");
        use.getParameter("artifact_name").setValue(aName);
        use.getParameter("sensor_name").setValue(sensorName);  
        
        sense = createGoal("sense");
        sense.getParameter("sensor_name").setValue(sensorName);
        sense.getParameter("filter").setValue("joined");
        sense.getParameter("timeout").setValue(1000);
        

        op = new Op("join", getAgentName() );
        use.getParameter("op").setValue(op);   
        
        dispatchSubgoalAndWait(use);        
        dispatchSubgoalAndWait(sense);
        
        // the environment artifact is signalling the tuple ("joined", START_LOCATION);;  
        p = (Perception)sense.getParameter("percept").getValue();       
        location myL = (location)p.getContent(0);
        log("my initial location is: " + myL);
        getBeliefbase().getBelief("my_loc").setFact(myL);
        
        //3. get environment dimensions
        
        use = createGoal("use");
        use.getParameter("artifact_name").setValue(aName);
        use.getParameter("sensor_name").setValue(sensorName);  
        op = new Op("getNRooms");
        use.getParameter("op").setValue(op); 
        
        sense = createGoal("sense");
        sense.getParameter("sensor_name").setValue(sensorName);
        sense.getParameter("filter").setValue("n_rooms");
        //sense.getParameter("timeout").setValue(1000); 
        
        dispatchSubgoalAndWait(use);        
        dispatchSubgoalAndWait(sense);

        // environment signals the tuple ("n_rooms", N_ROOMS); 
        p = (Perception)sense.getParameter("percept").getValue();       
        int NR = new Integer(""+p.getContent(0)).intValue();      
        log("RoomsWorld configured with : " + NR + " ROOMS...");
        getBeliefbase().getBelief("n_rooms").setFact(NR);
        
        //4. set up navigator component
        switch(NR) {
            case(4): navigator = new Rooms4(); break;
            case(8): navigator = new Rooms8(); break;
            default:  navigator = new Rooms4(); 
        }       
        getBeliefbase().getBelief("navigator").setFact(navigator); 
        
        IGoal selectRoom = createGoal("select_room");
        dispatchTopLevelGoal(selectRoom);
       
    }
    
    
        
    protected void log(String s){
        System.out.println("["+getAgentName()+"] " + s);
    }
    
}
