package roomsworld;

import alice.cartago.examples.roomsworld.*;
import alice.cartago.*;

import alice.cartago.jason.AgentContext;

import jason.JasonException;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.*;


import java.util.*;

/**
  <p>Internal action: <b><code>roomsword.goTo(p(<i>N</i>))</code></b>.
  
  <p>Description: the agent goes to a given location in <i>N</i>.
  
  <p>Parameter:<ul>
  
  <li>- value p(): p can be room(id) or checklist(id)<br/>
  <li>- value p(id): id is the index of the area (room or checklist)<br/>
  </ul>
  
  <p>Example:<ul> 

  <li><code>roomsword.goTo(checklist(<i>3</i>))</code> goes to checklist 3.</li>

  </ul>

*/

public class goTo extends alice.cartago.jason.CartagoAction {
    
    private int dest, myroom;
	private location myloc;
	private  Perception percepts;
	private  IRoomsWorldNavigator paths;
	
	private String agentName;
	private AgentContext actx;
	private ICartagoContext ctx;
	
	private ArtifactId aid;
	private SensorId sid;
	
	private   String myName = "Cleaner_Jason";
    private   String mySensorName = "mySensor";
	
    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {
		
		agentName = ts.getUserAgArch().getAgName();
		actx = getContext(agentName);
		ctx = actx.getContext();
        
		Term t;
		Structure s;
		
		try {
			// arg[0]  from-room			
            if (!args[0].isNumeric()) {
                throw new JasonException("The first argument of the internal action 'goTo' has to be numeric.");                
            }
			Double d = new Double(""+args[0]);
			double x1 = d.doubleValue();
			int myroom = d.intValue();
			
			// arg[1]  to-room
			t = (Term)args[1];
			if(t.isStructure()) {
				
				s = (Structure)t;
		        if (s.getTermsArray().length != 1) {
                   throw new JasonException("The first argument of the internal action 'goTo' has to be structure.");  
			    }else{
			    	// retrieve destination id
			    	dest = (int)((NumberTerm)s.getTerm(0)).solve(); 
			    }
		    	// retrieve destination type
		    	if (s.getFunctor().equals("room")) {
		    		goToRoom(dest);			
		    	}
		    	else if (s.getFunctor().equals("checklist")) {
			    	goToChecklist(dest);
		    	}
		    	else  {
		    		throw new JasonException("destination's functor does not match. types are checklist() or room()");  
		    	}
			}else if(t.isNumeric()) {
				double x2 = new Double(""+args[1]).doubleValue();
				goToXY(x1,x2);
	        } 
		
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new JasonException("The internal action 'goTo' has not received the required argument.");
        } catch (Exception e) {
            throw new JasonException("Error in internal action 'goTo': " + e, e);
        }    
		
		return true;
    }
	
	void goToRoom(int targetRoom) throws Exception {
		    
		    paths = new Rooms8();
			
			aid = getArtifactId("environment");
		    sid = getSensorId(mySensorName);
	
					 // get initial location
            retrieveLocation();

            System.out.println("my room is: " + myroom + " my location: " +myloc);
            System.out.println("New Intention - target room is: " + targetRoom);
           
            ArrayList<location> partialdests = paths.getPath(myroom,targetRoom);
            Iterator<location> itPoint = partialdests.iterator();
			
			while(itPoint.hasNext()){
                   goToLocation(itPoint.next());         
            } 
		}
		
	void goToChecklist(int targetCh) throws Exception {
		
		    paths = new Rooms8();
			
			aid = getArtifactId("environment");
		    sid = getSensorId(mySensorName);
			
			// get initial location
            retrieveLocation();
			
            myroom = paths.getRoom(myloc);
          
            System.out.println("my room is: " + myroom + "!!\n");
            System.out.println("New Intention - target checklist is: " + targetCh);
           
            ArrayList<location> partialdests = paths.getChPath(myroom,targetCh);
            Iterator<location> itPoint = partialdests.iterator();
			
			while(itPoint.hasNext()){
                   goToLocation(itPoint.next());         
            } 
		}
		
	void goToXY(double X, double Y) throws Exception {
		retrieveLocation();
		goToLocation(new location(X,Y));
	}
		
	void goToLocation(location target) throws Exception {
		
         while(!myloc.isEqual(target))   {
               // Ceck if location has changed in mean time.
               // calculate the new position offset.
               //double speed = 0.1;
           try {
               double d = myloc.getDistance(target);
               double r =  0.01;

               double dx = target.x-myloc.x;
               double dy = target.y-myloc.y;
               // When radius smaller than distance, just move a step.
               double rx = r<d ? r*dx/d : dx;
               double ry = r<d ? r*dy/d : dy;
               location newLoc = new location(myloc.x+rx, myloc.y+ry);
			   
			   //  ctx.use(aid,new Op(opName,opArgs),sid);	
			   // move(String agName, Object trg)
			   Op move = new Op("move", agentName, newLoc);
			   ctx.use(aid, move , sid);
			   String filter = "movedin"+newLoc.signalling();
               percepts = ctx.sense(sid, filter);
               myloc = (location)percepts.getContent(0); 
               //System.out.println("moved!");
 
              Thread.sleep(200);  //wait for 0.1 seconds_ this is speed....
           } catch(Exception e){ }

          } //while
     }
	 
	 void retrieveLocation() {
	   try{
		 ctx.use(aid, new Op("getLoc", agentName), sid);
		 String filter = agentName+"-loc";
         percepts = ctx.sense(sid, filter);
         myloc = (location)percepts.getContent(0); 
	     myroom = paths.getRoom(myloc);
	   } catch(Exception e){ e.printStackTrace(); }
	  }

	 
	 
	 SensorId getSensorId(String sname) throws JasonException{ 
		 SensorId sid = actx.getSensorId(sname);
		 if (sid==null){
				try {
					sid = ctx.getSensor(sname);
					actx.storeSensorId(sname,sid);
				} catch (Exception ex){
					throw new JasonException("use: impossible to exploit sensor : "+sname);
				}
			}
	     return sid;
	 }
	 
	 
	 ArtifactId getArtifactId(String artiName) throws JasonException{ 		 
	    return actx.getArtifactId(artiName);
	 
	 }
	
}

