
package roomsworld;

import jason.JasonException;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.NumberTermImpl;
import jason.asSyntax.Term;

import java.util.Random;

/**
  <p>Internal action: <b><code>roomsworld.randomInt(M, <i>N</i>)</code></b>.
  
  <p>Description: unifies <i>N</i> with an int random number between 0 and M.
  
  <p>Parameter:<ul>
  
  <li>- value (number): the variable to receive the random value<br/>
  <li>- value (number): the max integer of the interval<br/>
  </ul>
  
  <p>Example:<ul> 

  <li><code>roomsworld.random(M, X)</code>.</li>

  </ul>

*/
public class randomInt extends DefaultInternalAction {
    
    private Random random = new Random();    
	
    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {
        try {
			if (!args[0].isNumeric()) {
                throw new JasonException("The first argument of the internal action 'randomInt' has to be numeric.");                
            }
            if (!args[1].isVar()) {
                throw new JasonException("The second argument of the internal action 'random' is not a variable.");                
            }
			int R = random.nextInt(new Integer(""+args[0]).intValue() );
            return un.unifies(args[1], new NumberTermImpl(R));
			
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new JasonException("The internal action 'randomInt' has not received the required argument.");
        } catch (Exception e) {
            throw new JasonException("Error in internal action 'randomInt': " + e, e);
        }    
    }
}

