
package roomsworld;

import jason.JasonException;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.StringTermImpl;
import jason.asSyntax.Term;
import jason.asSyntax.Atom;


/**
  <p>Internal action: <b><code>roomsworld.toString(<i>St1, St2</i>, X)</code></b>.
  
  <p>Description: unifies <i>X</i> with St1 + St2 
  
  <p>Parameter:<ul>
  
  <li>- value (St1): first String<br/>
    <li>- value (St2): 2nd String<br/>
  <li>- value (X): the result concat. string St1+St2 <br/>
  </ul>
  
  </p>

*/
public class toString extends DefaultInternalAction {
       
	
    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {
        try {
			Term t0 = (Term)args[0];
			String one = getStringFromTerm(t0);
			
			Term t1 = (Term)args[1];
			String two = getStringFromTerm(t1);
			
			//if (!args[0].isString()) {
              //  throw new JasonException("The first argument of the internal action 'toString' has to be a String");                
            //}
            if (args[2].isVar()) {
               	String res = one+two;
                return un.unifies(args[2], new StringTermImpl(res));
			//                
            } else if (args[3].isVar()) {
				Term t3 = (Term)args[2];
				String three = getStringFromTerm(t3);
				String res = one+two+three;
				return un.unifies(args[3], new StringTermImpl(res));
			} else throw new JasonException("The 3rd argument of the internal action 'toString' has to be variable.");
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new JasonException("The internal action 'toString/3' has not received the required argument.");
        } catch (Exception e) {
            throw new JasonException("Error in internal action 'toString': " + e, e);
        }    
    }
	
	private String getStringFromTerm(Term t) throws Exception {
		if (t.isAtom()||t.isVar()||t.isStructure()||t.isNumeric() ){
			return t.toString();
		} else if (t.isString()){
			return (t.toString()).substring(1,t.toString().length()-1);
		} else {
			throw new JasonException("sense: invalid filter "+t);
		}
	}
}

