package roomsworld;

import alice.cartago.examples.roomsworld.*;
import alice.cartago.*;

import alice.cartago.jason.AgentContext;
import alice.cartago.jason.CartagoEnvironment;

import jason.JasonException;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.*;


import java.util.*;

/**
  <p>Internal action: <b><code>roomsword.epistemicAction(<i>N</i>)</code></b>.

  <p>Description: the agent get in vision objects in room <i>N</i>.

  <p>Parameter:<ul>

  <li>- value N: room id<br/>
  </ul>

  <p>Example:<ul>

  <li><code>roomsword.epistemicAction((<i>3</i>))</code> get in vision objects for rooms 3.</li>

  </ul>

*/

public class epistemicAction extends alice.cartago.jason.CartagoAction {

    private int dest, myroom;
	private location myloc;
	private  Perception percepts;
	private  IRoomsWorldNavigator paths;

	private String agentName;
	private AgentContext actx;
	private ICartagoContext ctx;

	private ArtifactId aid;
	private SensorId sid;

	private   String myName = "Cleaner_Jason";
    private   String mySensorName = "mySensor";

    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {

		agentName = ts.getUserAgArch().getAgName();
		actx = getContext(agentName);
		ctx = actx.getContext();
		aid = getArtifactId("environment");
		sid = getSensorId(mySensorName);

		int myroom;

		Term t;
		Structure s;

		try {
			// arg[0]  from-room
            if (!args[0].isNumeric()) {
                throw new JasonException("The first argument of the internal action 'epistemicAction' has to be numeric.");
            }
			myroom = new Integer(""+args[0]).intValue();



        } catch (ArrayIndexOutOfBoundsException e) {
            throw new JasonException("The internal action 'goTo' has not received the required argument.");
        } catch (Exception e) {
            throw new JasonException("Error in internal action 'goTo': " + e, e);
        }

		return getVision(ts, myroom-1);
    }

	boolean getVision(TransitionSystem ts, int nRoom) throws Exception {
		boolean ret = true;

	    Op getVision = new Op("getVision", agentName, nRoom);
					 // get initial location
		try{
			ctx.use(aid, getVision, sid);
            String filter = agentName+"vision";
            percepts = ctx.sense(sid, filter);
            Vision v = (Vision)percepts.getContent(0);
            
			ArrayList<trash> trashes = v.getTrashes();
            System.out.println("["+agentName+"] actual vision has: " + trashes.size() + " items");

		    if(trashes.size()>0){
		       Literal li;
		       for(int i=0; i<trashes.size();i++){
		    	   li = Literal.parseLiteral("trash("+ (nRoom+1) +", " + trashes.get(i).getLocation().getX()+","+ trashes.get(i).getLocation().getY() +")");
                   // add percept to the BeliefBase as *self* Beliefs
                   ret = ts.getAg().addBel(li);

                   // add percept to the BeliefBase as *percept* Events
                   //CartagoEnvironment.getInstance().addPercept(agentName, li);
		       }
			   if(!ret) throw new Exception(" error in adding percepts to the belief base");
			}
			
		} catch (Exception e) {
            throw new JasonException("Error in internal action 'roomsworld.epistemicAction': " + e, e);
        }

        return ret;
	}


	 SensorId getSensorId(String sname) throws JasonException{
		 SensorId sid = actx.getSensorId(sname);
		 if (sid==null){
				try {
					sid = ctx.getSensor(sname);
					actx.storeSensorId(sname,sid);
				} catch (Exception ex){
					throw new JasonException("use: impossible to exploit sensor : "+sname);
				}
			}
	     return sid;
	 }


	 ArtifactId getArtifactId(String artiName) throws JasonException{
	    return actx.getArtifactId(artiName);

	 }



}//epistemicAction

